@@ -2,6 +2,7 @@
 #include <inttypes.h>
 #include <stdio.h>
 #include <string.h>
+#include "../selector.h"
 
 #include <sys/types.h>
 
@@ -23,14 +24,21 @@ static id x(id self, SEL _cmd)
 
 int main(void)
 {
+
 	SEL nextSel;
 	Class cls = [Test class];
 	assert(cls != Nil);
 	int sel_size = 0;
 	for (uint32_t i=0 ; i<0xf0000 ; i++)
 	{
-		snprintf(selBuffer, 16, "%" PRId32 "selector%" PRIx32, i, i);
+		snprintf(selBuffer, sizeof(selBuffer), "%" PRId32 "selector%" PRIx32, i, i);
 		nextSel = sel_registerName(selBuffer);
+		char *registeredName = sel_getName(nextSel);
+		if (strcmp(selBuffer, registeredName) != 0)
+		{
+			fprintf(stderr, "'%s' != '%s' (%p)\n", selBuffer, registeredName, nextSel->index);
+		}
+		assert(strcmp(selBuffer, sel_getName(nextSel)) == 0);
 		sel_size += strlen(selBuffer);
 	}
 	assert(class_addMethod(object_getClass([Test class]), nextSel, (IMP)x, "@@:"));
@@ -39,8 +47,9 @@ int main(void)
 	objc_msg_lookup(cls, nextSel)(cls, nextSel);
 	assert(methodCalled == YES);
 	methodCalled = NO;
+#ifdef __GNUSTEP_MSGSEND__
 	objc_msgSend([Test class], nextSel);
 	assert(methodCalled == YES);
+#endif
 	return 0;
 }
-
