@@ -4,6 +4,7 @@
 #include "dwarf_eh.h"
 #include "objc/runtime.h"
 #include "objc/hooks.h"
+#include "objc/objc-exception.h"
 #include "class.h"
 #include "objcxx_eh.h"
 
@@ -20,12 +21,39 @@
 #define __builtin_unreachable abort
 #endif
 
+void test_cxx_eh_implementation();
+/**
+ * The Itanium C++ public structure for in-flight exception status.
+ */
+struct __cxa_eh_globals
+{
+	/**
+	 * The head exception object.  By convention, this is actually the end of
+	 * the `__cxa_exception` structure and points to the address of the thrown
+	 * object.  This is either an `id*` or a pointer to a C++ type that we're
+	 * not going to look at.
+	 */
+	struct __cxa_exception *caughtExceptions;
+	/**
+	 * The number of in-flight exceptions thrown.
+	 */
+	unsigned int uncaughtExceptions;
+};
+
+
+// Weak references to C++ runtime functions.  We don't bother testing that
+// these are 0 before calling them, because if they are not resolved then we
+// should not be in a code path that involves a C++ exception.
+__attribute__((weak)) void *__cxa_begin_catch(void *e);
+__attribute__((weak)) void __cxa_end_catch(void);
+__attribute__((weak)) void __cxa_rethrow(void);
+__attribute__((weak)) struct __cxa_eh_globals *__cxa_get_globals(void);
+
 
 /**
  * Class of exceptions to distinguish between this and other exception types.
  */
 static const uint64_t objc_exception_class = EXCEPTION_CLASS('G','N','U','C','O','B','J','C');
-static const uint64_t cxx_exception_class = EXCEPTION_CLASS('G','N','U','C','C','+','+','\0');
 
 /**
  * Structure used as a header on thrown exceptions.  
@@ -74,6 +102,34 @@ typedef enum
 	handler_class
 } handler_type;
 
+enum exception_type
+{
+	NONE,
+	CXX,
+	OBJC,
+	FOREIGN,
+	BOXED_FOREIGN
+};
+struct thread_data
+{
+	enum exception_type current_exception_type;
+	BOOL cxxCaughtException;
+	struct objc_exception *caughtExceptions;
+};
+
+static __thread struct thread_data thread_data;
+
+static struct thread_data *get_thread_data(void)
+{
+	return &thread_data;
+}
+
+static struct thread_data *get_thread_data_fast(void)
+{
+	return &thread_data;
+}
+
+
 /**
  * Saves the result of the landing pad that we have found.  For ARM, this is
  * stored in the generic unwind structure, while on other platforms it is
@@ -146,6 +202,9 @@ static void cleanup(_Unwind_Reason_Code reason, struct _Unwind_Exception *e)
 					unwindHeader)));
 					*/
 }
+
+void objc_exception_rethrow(struct _Unwind_Exception *e);
+
 /**
  * Throws an Objective-C exception.  This function is, unfortunately, used for
  * rethrowing caught exceptions too, even in @finally() blocks.  Unfortunately,
@@ -153,6 +212,22 @@ static void cleanup(_Unwind_Reason_Code reason, struct _Unwind_Exception *e)
  */
 void objc_exception_throw(id object)
 {
+	struct thread_data *td = get_thread_data();
+	DEBUG_LOG("Throwing %p, in flight exception: %p\n", object, td->lastThrownObject);
+	DEBUG_LOG("Exception caught by C++: %d\n", td->cxxCaughtException);
+	// If C++ caught the exception, then we may need to make C++ rethrow it if
+	// we want to preserve exception state.  Rethrows should be handled with
+	// objc_exception_rethrow, but clang appears to do the wrong thing for some
+	// cases.
+	if (td->cxxCaughtException)
+	{
+		struct __cxa_eh_globals *globals = __cxa_get_globals();
+		if ((globals->caughtExceptions != NULL) &&
+		    (*(id*)globals->caughtExceptions == object))
+		{
+			__cxa_rethrow();
+		}
+	}
 
 	SEL rethrow_sel = sel_registerName("rethrow");
 	if ((nil != object) &&
@@ -174,6 +249,8 @@ void objc_exception_throw(id object)
 
 	ex->object = object;
 
+	td->cxxCaughtException = NO;
+
 	_Unwind_Reason_Code err = _Unwind_RaiseException(&ex->unwindHeader);
 	free(ex);
 	if (_URC_END_OF_STACK == err && 0 != _objc_unexpected_exception)
@@ -327,6 +404,11 @@ static inline _Unwind_Reason_Code internal_objc_personality(int version,
 	void *object = NULL;
 
 #ifndef NO_OBJCXX
+	if (cxx_exception_class == 0)
+	{
+		test_cxx_eh_implementation();
+	}
+
 	if (exceptionClass == cxx_exception_class)
 	{
 		int objcxx;
@@ -468,80 +550,64 @@ static inline _Unwind_Reason_Code internal_objc_personality(int version,
 		}
 	}
 
-	_Unwind_SetIP(context, (unsigned long)action.landing_pad);
+	_Unwind_SetIP(context, (uintptr_t)action.landing_pad);
 	_Unwind_SetGR(context, __builtin_eh_return_data_regno(0), 
-			(unsigned long)(isNew ? exceptionObject : object));
+			(uintptr_t)(isNew ? exceptionObject : object));
 	_Unwind_SetGR(context, __builtin_eh_return_data_regno(1), selector);
 
 	DEBUG_LOG("Installing context, selector %d\n", (int)selector);
+	get_thread_data()->cxxCaughtException = NO;
 	return _URC_INSTALL_CONTEXT;
 }
 
+OBJC_PUBLIC
 BEGIN_PERSONALITY_FUNCTION(__gnu_objc_personality_v0)
 	return internal_objc_personality(version, actions, exceptionClass,
 			exceptionObject, context, NO);
 }
+
+OBJC_PUBLIC
 BEGIN_PERSONALITY_FUNCTION(__gnustep_objc_personality_v0)
 	return internal_objc_personality(version, actions, exceptionClass,
 			exceptionObject, context, YES);
 }
 
+OBJC_PUBLIC
 BEGIN_PERSONALITY_FUNCTION(__gnustep_objcxx_personality_v0)
+#ifndef NO_OBJCXX
+	if (cxx_exception_class == 0)
+	{
+		test_cxx_eh_implementation();
+	}
 	if (exceptionClass == objc_exception_class)
 	{
 		struct objc_exception *ex = objc_exception_from_header(exceptionObject);
 		if (0 == ex->cxx_exception)
 		{
-			id *newEx = __cxa_allocate_exception(sizeof(id));
-			*newEx = ex->object;
-			ex->cxx_exception = objc_init_cxx_exception(newEx);
-			memcpy(ex->cxx_exception, exceptionObject, sizeof(struct _Unwind_Exception));
-			ex->cxx_exception->exception_class = cxx_exception_class;
-			ex->cxx_exception->exception_cleanup = cleanup;
+			ex->cxx_exception = objc_init_cxx_exception(ex->object);
 		}
+		// We now have two copies of the _Unwind_Exception object (which stores
+		// state for the unwinder) in flight.  Make sure that they're in sync.
+		COPY_EXCEPTION(ex->cxx_exception, exceptionObject);
 		exceptionObject = ex->cxx_exception;
 		exceptionClass = cxx_exception_class;
+		int ret = CALL_PERSONALITY_FUNCTION(__gxx_personality_v0);
+		COPY_EXCEPTION(exceptionObject, ex->cxx_exception);
+		if (ret == _URC_INSTALL_CONTEXT)
+		{
+			get_thread_data()->cxxCaughtException = YES;
+		}
+		return ret;
 	}
+#endif
 	return CALL_PERSONALITY_FUNCTION(__gxx_personality_v0);
 }
 
-// Weak references to C++ runtime functions.  We don't bother testing that
-// these are 0 before calling them, because if they are not resolved then we
-// should not be in a code path that involves a C++ exception.
-__attribute__((weak)) void *__cxa_begin_catch(void *e);
-__attribute__((weak)) void __cxa_end_catch(void);
-__attribute__((weak)) void __cxa_rethrow(void);
-
-enum exception_type
-{
-	NONE,
-	CXX,
-	OBJC,
-	FOREIGN,
-	BOXED_FOREIGN
-};
-struct thread_data
-{
-	enum exception_type current_exception_type;
-	struct objc_exception *caughtExceptions;
-};
-
-static __thread struct thread_data thread_data;
-
-static struct thread_data *get_thread_data(void)
-{
-	return &thread_data;
-}
-
-static struct thread_data *get_thread_data_fast(void)
-{
-	return &thread_data;
-}
-
-id objc_begin_catch(struct _Unwind_Exception *exceptionObject)
+OBJC_PUBLIC id objc_begin_catch(struct _Unwind_Exception *exceptionObject)
 {
 	struct thread_data *td = get_thread_data();
 	DEBUG_LOG("Beginning catch %p\n", exceptionObject);
+	td->cxxCaughtException = NO;
 	if (exceptionObject->exception_class == objc_exception_class)
 	{
 		td->current_exception_type = OBJC;
@@ -576,6 +642,7 @@ id objc_begin_catch(struct _Unwind_Exception *exceptionObject)
 		// exceptions are in-flight
 		abort();
 	}
+#ifndef NO_OBJCXX
 	// If this is a C++ exception, let the C++ runtime handle it.
 	if (exceptionObject->exception_class == cxx_exception_class)
 	{
@@ -583,6 +650,7 @@ id objc_begin_catch(struct _Unwind_Exception *exceptionObject)
 		td->current_exception_type = CXX;
 		return __cxa_begin_catch(exceptionObject);
 	}
+#endif
 	DEBUG_LOG("foreign exception catch\n");
 	// Box if we have a boxing function.
 	if (_objc_class_for_boxing_foreign_exception)
@@ -607,7 +675,7 @@ id objc_begin_catch(struct _Unwind_Exception *exceptionObject)
 	return (id)((char*)exceptionObject + sizeof(struct _Unwind_Exception));
 }
 
-void objc_end_catch(void)
+OBJC_PUBLIC void objc_end_catch(void)
 {
 	struct thread_data *td = get_thread_data_fast();
 	// If this is a boxed foreign exception then the boxing class is
@@ -653,7 +721,7 @@ void objc_end_catch(void)
 	}
 }
 
-void objc_exception_rethrow(struct _Unwind_Exception *e)
+OBJC_PUBLIC void objc_exception_rethrow(struct _Unwind_Exception *e)
 {
 	struct thread_data *td = get_thread_data_fast();
 	// If this is an Objective-C exception, then 
@@ -674,11 +742,13 @@ void objc_exception_rethrow(struct _Unwind_Exception *e)
 		}
 		abort();
 	}
+#ifndef NO_OBJCXX
 	else if (td->current_exception_type == CXX)
 	{
 		assert(e->exception_class == cxx_exception_class);
 		__cxa_rethrow();
 	}
+#endif
 	if (td->current_exception_type == BOXED_FOREIGN)
 	{
 		SEL rethrow_sel = sel_registerName("rethrow");
@@ -695,3 +765,8 @@ void objc_exception_rethrow(struct _Unwind_Exception *e)
 	_Unwind_Resume_or_Rethrow(e);
 	abort();
 }
+
+objc_uncaught_exception_handler objc_setUncaughtExceptionHandler(objc_uncaught_exception_handler handler)
+{
+	return __atomic_exchange_n(&_objc_unexpected_exception, handler, __ATOMIC_SEQ_CST);
+}
