@@ -9,13 +9,14 @@ typedef struct _NSZone NSZone;
 @end
 #include <stdio.h>
 
-/**
- * Equivalent to [cls alloc].  If there's a fast path opt-in, then this skips the message send.
- */
 OBJC_PUBLIC
 id
 objc_alloc(Class cls)
 {
+	if (UNLIKELY(cls == nil))
+	{
+		return nil;
+	}
 	if (UNLIKELY(!objc_test_class_flag(cls->isa, objc_class_flag_initialized)))
 	{
 		objc_send_initialize(cls);
@@ -34,6 +35,10 @@ OBJC_PUBLIC
 id
 objc_allocWithZone(Class cls)
 {
+	if (UNLIKELY(cls == nil))
+	{
+		return nil;
+	}
 	if (UNLIKELY(!objc_test_class_flag(cls->isa, objc_class_flag_initialized)))
 	{
 		objc_send_initialize(cls);
@@ -53,7 +58,14 @@ OBJC_PUBLIC
 id
 objc_alloc_init(Class cls)
 {
+	if (UNLIKELY(cls == nil))
+	{
+		return nil;
+	}
 	id instance = objc_alloc(cls);
+	// If +alloc was overwritten, it is not guaranteed that it returns
+	// an instance of cls.
+	cls = classForObject(instance);
 	if (objc_test_class_flag(cls, objc_class_flag_fast_alloc_init))
 	{
 		return instance;
