@@ -1,4 +1,3 @@
 #include "objc/runtime.h"
-#define OBJC_HOOK
+#define OBJC_HOOK OBJC_PUBLIC
 #include "objc/hooks.h"
-
