@@ -2,6 +2,7 @@
 #pragma clang system_header
 #endif
 #include "objc-visibility.h"
+#include "objc-config.h"
 
 #ifndef __LIBOBJC_RUNTIME_H_INCLUDED__
 #define __LIBOBJC_RUNTIME_H_INCLUDED__
@@ -26,6 +27,12 @@ extern "C" {
 #	endif
 #endif
 
+#if defined(__OBJC__) && __has_attribute(ns_returns_retained)
+#	define OBJC_RETURNS_RETAINED __attribute__((ns_returns_retained))
+#else
+#	define OBJC_RETURNS_RETAINED
+#endif
+
 // Make sure we get the limit macros, even in C++ mode
 #ifndef __STDC_LIMIT_MACROS                                                     
 #	define __STDC_LIMIT_MACROS 1
@@ -190,30 +197,32 @@ typedef struct
 
 
 #ifndef YES
-#	define YES ((BOOL)1)
+#	if __has_feature(objc_bool)
+#		define YES __objc_yes
+#	else
+#		define YES ((BOOL)1)
+#	endif
 #endif
 #ifndef NO
-#	define NO ((BOOL)0)
+#	if __has_feature(objc_bool)
+#		define NO __objc_no
+#	else
+#		define NO ((BOOL)0)
+#	endif
 #endif
 
-#ifdef __GNUC
-#	define _OBJC_NULL_PTR __null
-#elif defined(__cplusplus)
-#	if __has_feature(cxx_nullptr)
-#		define _OBJC_NULL_PTR nullptr
-#	else
-#		define _OBJC_NULL_PTR 0
-#	endif
+#if __has_feature(cxx_nullptr)
+#	define _OBJC_NULL_PTR nullptr
 #else
-#	define _OBJC_NULL_PTR ((void*)0)
+#	define _OBJC_NULL_PTR NULL
 #endif
 
 #ifndef nil
-#	define nil ((id)_OBJC_NULL_PTR)
+#	define nil _OBJC_NULL_PTR
 #endif
 
 #ifndef Nil
-#	define Nil ((Class)_OBJC_NULL_PTR)
+#	define Nil _OBJC_NULL_PTR
 #endif
 
 #include "slot.h"
@@ -287,7 +296,7 @@ Protocol *__unsafe_unretained* class_copyProtocolList(Class cls, unsigned int *o
 /**
  * Creates an instance of this class, allocating memory using malloc.
  */
-OBJC_PUBLIC
+OBJC_PUBLIC OBJC_RETURNS_RETAINED
 id class_createInstance(Class cls, size_t extraBytes);
 
 /**
@@ -1054,7 +1063,7 @@ typedef uintptr_t objc_AssociationPolicy;
  * with the same arguments, or nil if none exists.
  */
 OBJC_PUBLIC
-id objc_getAssociatedObject(id object, void *key);
+id objc_getAssociatedObject(id object, const void *key);
 /**
  * Associates an object with another.  This provides a mechanism for storing
  * extra state with an object, beyond its declared instance variables.  The
@@ -1064,7 +1073,7 @@ id objc_getAssociatedObject(id object, void *key);
  * if an association policy of copy or retain is passed as the final argument.
  */
 OBJC_PUBLIC
-void objc_setAssociatedObject(id object, void *key, id value, objc_AssociationPolicy policy);
+void objc_setAssociatedObject(id object, const void *key, id value, objc_AssociationPolicy policy);
 /**
  * Removes all associations from an object.  
  */
@@ -1077,7 +1086,7 @@ void objc_removeAssociatedObjects(id object);
  * arguments as the method.
  */
 OBJC_PUBLIC
-IMP imp_implementationWithBlock(void *block);
+IMP imp_implementationWithBlock(id block);
 /**
  * Returns the type encoding of an IMP that would be returned by passing the
  * block to imp_implementationWithBlock().  Returns NULL if this is not a valid
@@ -1085,14 +1094,14 @@ IMP imp_implementationWithBlock(void *block);
  * argument).  The caller is responsible for freeing the returned value.
  */
 OBJC_PUBLIC
-char *block_copyIMPTypeEncoding_np(void*block);
+char *block_copyIMPTypeEncoding_np(id block);
 /**
  * Returns the block that was used in an IMP created by
  * imp_implementationWithBlock().  The result of calling this function with any
  * other IMP is undefined.
  */
 OBJC_PUBLIC
-void *imp_getBlock(IMP anImp);
+id imp_getBlock(IMP anImp);
 /**
  * Removes a block that was converted to an IMP with
  * imp_implementationWithBlock().  The result of calling this function with any
@@ -1150,6 +1159,22 @@ id object_getPrototype_np(id object);
 OBJC_PUBLIC
 int objc_set_apple_compatible_objcxx_exceptions(int newValue) OBJC_NONPORTABLE;
 
+/** 
+ * This function is inserted by the compiler when a mutation is detected during
+ * a foreach iteration. It is exported as a weak symbol to enable GNUstep or
+ * some other framework to replace it trivially.
+ */
+OBJC_PUBLIC
+void __attribute__((weak)) objc_enumerationMutation(id obj);
+
+/**
+ * Ensure that `+initialize` has been sent to the class of the argument (or the
+ * argument, if it is a class).  This will not call `+initialize` if it has
+ * been called already, either via an explicit call to this function or by
+ * being sent some other message.
+ */
+OBJC_PUBLIC
+void objc_send_initialize(id object) OBJC_NONPORTABLE;
 
 #define _C_ID       '@'
 #define _C_CLASS    '#'
